# -*- coding: utf-8 -*-
"""
    Nouveaux tests pour Eole-2.3
    Utilisation du dictionnaire : data/eole23.xml
    Tests de non régression pour les demandes :
    #1794, #1738, #1344, #3039
"""
from py.test import raises
from os import path as osp
from creole.cfgparser import EoleDict
from creole import process

HERE = osp.join(osp.dirname(__file__))
DATADIR = osp.join(HERE, 'data')

class Test23:

    def setup_class(self):
        """
        chargement du dictionnaire
        """
        self.dico = EoleDict(templatedir=DATADIR, logfile='/dev/null')
        self.dico.read(osp.join(DATADIR, 'eole23.xml'))

    def test_simplefill(self):
        """
        test d'une variable "simple" avec un fill
        """
        assert self.dico.variables['value'].get_value() == [u'valeur']
        # init_fills a été appelé à la création de la variable
        assert self.dico.variables['simplefill'].val == [u'valeur']
        assert self.dico.variables['simplefill'].get_value() == [u'valeur']
        self.dico.save_values('/dev/null')
        # pas de modif après enregistrement
        assert self.dico.variables['simplefill'].get_value() == [u'valeur']
        self.dico.variables['simplefill'].set_value('toto')
        assert self.dico.variables['simplefill'].get_value() == [u'toto']
        self.dico.variables['simplefill'].set_value('')
        self.dico.save_values('/dev/null')
        # calcul fait au save_value
        assert self.dico.variables['simplefill'].val == [u'valeur']

    def test_modfill(self):
        """
        test avec modification de la variable utilisée pour calc_val
        """
        self.dico.variables['value'].set_value('valeur2')
        # le fill n'est pas re-calculé
        assert self.dico.variables['simplefill'].val == [u'valeur']
        # ré-initialisation "manuelle"
        self.dico.variables['simplefill'].set_value('')
        assert self.dico.variables['simplefill'].val == [u'']
        assert self.dico.variables['simplefill'].get_value() == [u'valeur2']
        # get_value() ne fait que lire
        assert self.dico.variables['simplefill'].val == [u'']
        self.dico.save_values('/dev/null')
        assert self.dico.variables['simplefill'].val == [u'valeur2']

    def test_group_default_values(self):
        """
        test sur un groupe de variables
        avec des variables esclaves ayant différentes valeurs par défaut
        """
        assert self.dico.variables['master'].get_value() == ['test', 'test2']
        assert self.dico.variables['slave1'].get_value() == ['a', 'b']
        # les valeurs par défaut sont dupliquées si nécessaire
        assert self.dico.variables['slave2'].get_value() == ['1', '1']
        assert self.dico.variables['slave3'].get_value() == ['', '']
        assert self.dico.variables['slave4'].get_value() == ['', '']
        # => l'ensemble est valide
        assert self.dico.variables['master'].get_final_value() == ['test', 'test2']

    def test_group_default_no_master_value(self):
        """
        test sur un groupe de variables
        avec des variables esclaves ayant différentes valeurs par défaut
        la variable maître n'a pas de valeur
        """
        assert self.dico.variables['slave2_1'].get_value() == ['1']

    def test_group_new_slaves(self):
        """
        test sur un groupe de variable
        avec ajout d'une valeur sur la variables maître
        """
        self.dico.variables['master'].set_value(['x', 'y', 'z'])
        # slave1 a deux valeurs par défaut => levée d'exception
        raises(AssertionError, """self.dico.variables['master'].get_final_value()""")
        self.dico.variables['slave1'].set_value(['a', 'b', 'c'])
        # les valeurs par défaut sont dupliquées si nécessaire
        assert self.dico.variables['slave2'].get_value() == ['1', '1', '1']
        assert self.dico.variables['slave3'].get_value() == ['', '', '']
        assert self.dico.variables['slave4'].get_value() == ['', '', '']
        # => l'ensemble est valide
        assert self.dico.variables['master'].get_final_value() == ['x', 'y', 'z']

    def test_simple_multi(self):
        """
        test d'une variable multi non obligatoire
        et sans esclaves
        """
        assert self.dico.variables['multi'].get_final_value() == []
        self.dico.variables['multi'].set_value('test')
        self.dico.save_values('/dev/null')
        assert self.dico.variables['multi'].get_final_value() == [u'test']

    def test_multi_hidden(self):
        """
        une variable cachée en multi doit renvoyer sa valeur (#1197)
        """
        assert self.dico.variables['multi_hidden'].get_value() == [u'toto']
        assert self.dico.variables['multi_hidden'].get_final_value() == [u'toto']

class Test23_6:

    def setup_class(self):
        """
        chargement du dictionnaire
        """
        self.dico = EoleDict(templatedir=DATADIR, logfile='/dev/null')
        self.dico.read(osp.join(DATADIR, 'eole23_6.xml'))

    def test_group_new_slaves(self):
        """
        test sur un groupe de variable
        ajout d'une slave avec un fill sur une variable non multi
        et d'un slave avec un fill sur une variable multi
        """
        assert self.dico.variables['slave'].get_value() == ['']

        self.dico.variables['master'].set_value(['x', 'y', 'z'])
        assert self.dico.variables['slave'].get_value() == ['', '', '']
        self.dico.variables['simple'].set_value(['x'])
        assert self.dico.variables['slave'].get_value() == ['x', 'x', 'x']

